"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Root_1 = require("./Root");
const libs_1 = require("../libs");
const symbols_1 = require("../symbols");
const Omit = require("omit-empty");
class Data extends Root_1.Root {
    /**
     * ClassData constructor loads a initial state of the object, if needed.
     * @param initialData
     */
    constructor(initialData = {}) {
        super();
        this
            .assign(initialData)
            .resetDirtyState();
    }
    assign(initialData) {
        const deserialized = this.deserialize(initialData);
        Object.assign(this, deserialized);
        return this;
    }
    set(attributes) {
        const deserialized = this.deserialize(attributes);
        Object.assign(this, Omit(deserialized));
        return this;
    }
    /**
     * Loop through each data attribute of the class an run
     * a serialization function on the value, if provided
     */
    serialize() {
        this.debug('::serialize', 'airboat:classes:data');
        const result = {};
        symbols_1.initClassDataSymbols(this);
        Object.keys(this[symbols_1.CLASS_DATA_ATTRIBUTES]).forEach((key) => {
            const attribute = this[symbols_1.CLASS_DATA_ATTRIBUTES][key];
            if (this[attribute.name] !== undefined) {
                if (attribute.serialize) {
                    result[attribute.name] = attribute.serialize(this[attribute.name]);
                }
                else {
                    result[attribute.name] = this[attribute.name];
                }
            }
        });
        return Omit(result);
    }
    /**
     * Loop through each key of a given object, and will try to load
     * the values in the object if their keys is found in the data class
     * attribute list. If provided, will run a deserialization function
     * on each value.
     *
     * @param object
     */
    deserialize(object) {
        this.debug('::deserialize', 'airboat:classes:data');
        const deserialized = {};
        symbols_1.initClassDataSymbols(this);
        Object.keys(this[symbols_1.CLASS_DATA_ATTRIBUTES]).forEach((attrKey) => {
            const attrDef = this[symbols_1.CLASS_DATA_ATTRIBUTES][attrKey];
            if (attrDef.deserialize) {
                // A deserialize function is given, always use it
                deserialized[attrKey] = attrDef.deserialize(object[attrKey]);
            }
            else {
                // No deserialize function, simply return the value
                deserialized[attrKey] = object[attrKey];
            }
        });
        return deserialized;
    }
    /**
     * Deep check if an attribute exists. Please refer to lodash for more details.
     *
     * @param key
     */
    has(key) {
        return libs_1._.has(this, key);
    }
    /**
     * Creates an object composed of the picked object properties.
     *
     * @param paths
     */
    pick(paths) {
        return libs_1._.pick(this.serialize(), paths);
    }
    /**
     * Gets the value at `path`. Returns `defaultValue` or false if nothing is found.
     *
     * @param path
     * @param defaultValue
     */
    get(path, defaultValue = false) {
        return (libs_1._.get(this, path, defaultValue));
    }
    /**
     * Checks if the value at `path` equals `value`
     *
     * @param path
     * @param value
     */
    is(path, value) {
        return this.get(path) === value;
    }
    /**
     * The opposite of `pick`; this method creates an object composed of the own and inherited enumerable property paths that are not omitted.
     *
     * @param paths
     */
    omit(paths) {
        return libs_1._.omit(this.serialize(), paths);
    }
    toJSON() {
        return this.serialize();
    }
    toString() {
        return JSON.stringify(this.serialize(), null, 2);
    }
    resetDirtyState() {
        this.initialData = this.serialize();
        return this;
    }
    getDirtyAttributes() {
        const dirtyAttributes = [];
        Object.keys(this[symbols_1.CLASS_DATA_ATTRIBUTES]).forEach((key) => {
            const attribute = this[symbols_1.CLASS_DATA_ATTRIBUTES][key];
            const currentValues = this.serialize();
            let initialValue = this.initialData[attribute.name];
            let currentValue = currentValues[attribute.name];
            if (initialValue && typeof initialValue.serialize === 'function') {
                initialValue = initialValue.serialize();
            }
            if (currentValue && typeof currentValue.serialize === 'function') {
                currentValue = currentValue.serialize();
            }
            if (!libs_1._.isEqual(initialValue, currentValue)) {
                dirtyAttributes.push(attribute.name);
            }
        });
        return dirtyAttributes;
    }
}
exports.Data = Data;
//# sourceMappingURL=Data.js.map